@@ -172,8 +172,8 @@ copyRequireAndDeps <- function(RPackageFolders, verbose = getOption("Require.ver
       if (isTRUE(pkgInstalledAlready)) {
         fromFiles <- dir(thePath, recursive = TRUE, full.names = TRUE)
         if (!newPathExists) {
-          message("Placing copy of ", pkg, " in ", RPackageFolders,
-                  verbose = verbose, verboseLevel = 1)
+          messageVerbose("Placing copy of ", pkg, " in ", RPackageFolders,
+                  verbose = verbose, verboseLevel = 2)
           dirs <- unique(dirname(fromFiles))
           dirs <- gsub(thePath, theNewPath, dirs)
           lapply(dirs, checkPath, create = TRUE)
@@ -199,20 +199,28 @@ copyRequireAndDeps <- function(RPackageFolders, verbose = getOption("Require.ver
 
 #' Setup for binary Linux repositories
 #'
-#' Enable use of binary package builds for Linux from the Rstudio Package Manager repo.
-#' This will set the `repos` option, affecting the current R session.
+#' Enable use of binary package builds for Linux from the RStudio Package Manager repo.
+#' This will set the `repos` option, affecting the current R session. It will put this
+#' `binaryLinux` in the first position. If the `getOption("repos")` is `NULL`, it will
+#' put `backupCRAN` in second position.
 #'
 #' @param binaryLinux A CRAN repository serving binary Linux packages.
+#' @param backupCRAN If there is no CRAN repository set
 #'
 #' @export
-setLinuxBinaryRepo <- function(binaryLinux = "https://packagemanager.rstudio.com/") {
+setLinuxBinaryRepo <- function(binaryLinux = "https://packagemanager.rstudio.com/",
+                               backupCRAN = srcPackageURLOnCRAN) {
   if (Sys.info()["sysname"] == "Linux" && grepl("Ubuntu", utils::osVersion)) {
     if (!grepl("R Under development", R.version.string) && getRversion() >= "4.1") {
-      options(
-        repos = c(CRAN =
-          paste0(binaryLinux, "all/__linux__/", system("lsb_release -cs", intern = TRUE), "/latest")
-        )
-      )
+      repo <- c(CRAN =
+                  paste0(binaryLinux, "all/__linux__/", system("lsb_release -cs", intern = TRUE), "/latest"))
+      if (!is.null(getOption("repos"))) {
+        backupCRAN <- getOption("repos")
+      }
+      if (is.null(names(backupCRAN))) names(backupCRAN) <- "CRAN"
+      repo <- c(repo, backupCRAN)
+      repo <- repo[!duplicated(repo)]
+      options(repos = repo)
     }
   }
 }
