@@ -16,6 +16,8 @@ utils::globalVariables(c(
       list2env(pkgEnvLast, .pkgEnv)
     }
   }
+  if (!is.null(getOptionRPackageCache()))
+    dir.create(getOptionRPackageCache(), showWarnings = FALSE, recursive = TRUE)
 
   invisible()
 }
@@ -24,7 +26,8 @@ utils::globalVariables(c(
   if (isInteractive()) {
     mess <- c(
       "Require version: ", as.character(utils::packageVersion("Require")), ".\n",
-      "  Using cache directory: ", RequireCacheDir(), ".\n",
+      if (!is.null(getOptionRPackageCache()))
+        paste0("  Using cache directory: ", getOptionRPackageCache(), ".\n"),
       "  See ?RequireOptions for additional settings."
     )
 
