@@ -19,4 +19,5 @@ tryCatch(checkPath(tmpdir, create = FALSE), error = function(e) FALSE) ## FALSE
 checkPath(tmpdir, create = TRUE)
 dir.exists(tmpdir) ## TRUE
 
-unlink(tmpdir, recursive = TRUE)
+unlink(tmpdir, recursive = TRUE) # clean up
+
