@@ -4,6 +4,8 @@ tdOuter <- tempdir2("tests")
 try(saveRDS(startTimeAll, file = file.path(tdOuter, "startTimeAll")), silent = TRUE)
 message("\033[32m --------------------------------- Starting ", thisFilename, "  at: ",
         format(startTime),"---------------------------\033[39m")
+messageVerbose("\033[34m getOption('Require.verbose'): ", getOption("Require.verbose"), "\033[39m", verboseLevel = 0)
+messageVerbose("\033[34m getOption('repos'): ", paste(getOption("repos"), collapse = ", "), "\033[39m", verboseLevel = 0)
 
 library(Require)
 srch <- search()
@@ -11,18 +13,45 @@ anyNamespaces <- srch[!gsub("package:", "", srch) %in%
                         c("Require", Require:::.basePkgs, ".GlobalEnv", "tools:rstudio", "Autoloads", "testit")]
 if (length(anyNamespaces) > 0) stop("Please restart R before running this test")
 library(testit)
+
 origLibPathsAllTests <- .libPaths()
-tmpdir <- checkPath(tempdir2("testingA"), create = TRUE)
+
+# tmpdir <- tempdir2(Require:::.rndstr(1))
+# created <- dir.create(tmpdir, recursive = TRUE, showWarnings = FALSE)
+#
+# .libPaths(tmpdir)
+# Require("covr (==3.5.0)", require = FALSE)
+# pkgVF <- file.path(tmpdir, "packageVersions.txt")
+# aa <- pkgSnapshot(packageVersionFile = pkgVF, libPaths = .libPaths()[1])
+
+tmpdir <- tempdir2(Require:::.rndstr(1))
+tmpdir2 <- tempdir2(Require:::.rndstr(1))
+created <- dir.create(tmpdir, recursive = TRUE, showWarnings = FALSE)
 pkgVF <- file.path(tmpdir, "packageVersions.txt")
-aa <- pkgSnapshot(packageVersionFile = pkgVF)
+setLibPaths(tmpdir, standAlone = TRUE)
+tmpdirActual <- .libPaths()[1] # setLibPaths postpends the R version
+Require(c("remotes (==2.4.1)", "testit (==0.12)"), require = FALSE)
+
+setLibPaths(tmpdir2, standAlone = TRUE)
+tmpdir2Actual <- .libPaths()[1] # setLibPaths postpends the R version
+Require(c("covr (==3.5.0)"), require = FALSE)
+
+.libPaths(c(tmpdirActual, tmpdir2Actual))
+# .libPaths(c(tmpdir, tmpdir2))
+aa <- pkgSnapshot(packageVersionFile = pkgVF, libPaths = .libPaths()[1:2])
+
 bb <- list()
 for (lp in unique(aa$LibPath)) {
   pack <- aa$Package[aa$LibPath == lp]
+  pack <- sample(pack, size = min(10, length(pack)))
   if (!all(pack %in% Require:::.basePkgs)) {
     deps <- pkgDep(pack, recursive = TRUE)
-    haveNoDeps <- unlist(lapply(deps, length)) == 2
-    sam <- sample(sum(haveNoDeps), pmin(sum(haveNoDeps), 3))
-    pkgs <- c(names(deps[haveNoDeps][sam]), Require::extractPkgName(unname(unlist(deps[haveNoDeps][sam]))))
+    lens <- lengths(deps)
+    haveFewDeps <- order(lens)
+    deps <- deps[haveFewDeps]
+    wh <- min(4, max(which(cumsum(lengths(deps)+1) < 10)))
+    deps <- deps[seq(wh)]
+    pkgs <- c(names(deps), Require::extractPkgName(unname(unlist(deps))))
     bb[[lp]] <- aa[Package %in% pkgs & LibPath == lp]
   }
 }
@@ -33,9 +62,7 @@ outOpts <- options(
   install.packages.check.source = "never",
   install.packages.compile.from.source = "never",
   Require.persistentPkgEnv = TRUE,
-  Require.unloadNamespaces = TRUE,
-  Require.verbose = FALSE
-)
+  Require.unloadNamespaces = TRUE)
 if (Sys.info()["user"] == "achubaty") {
   outOpts2 <- options("Require.Home" = "~/GitHub/PredictiveEcology/Require")
 } else {
@@ -47,7 +74,7 @@ if (file.exists(pkgVF)) {
   baseFN <- "packageVersions"
   tmpLibPath <- tempdir2(paste(sample(LETTERS, size = 6), collapse = ""))
   fileNames[["fn0"]][["lp"]] <- file.path(tmpLibPath)
-  fileNames[["fn0"]][["txt"]] <- file.path(tmpdir, paste0(baseFN, ".txt"))
+  fileNames[["fn0"]][["txt"]] <- pkgVF
 
   try(setLibPaths(origLibPaths[[1]], updateRprofile = FALSE), silent = TRUE)
   Sys.setenv("R_REMOTES_UPGRADE" = "never")
@@ -77,13 +104,13 @@ if (file.exists(pkgVF)) {
     dts <- grep("data.table", localBinsFull, value = TRUE)[1]
     # rems <- grep("remotes", localBinsFull, value = TRUE)[1]
     localBinsFull <- na.omit(c(dts))#, rems))
+    # dts <- grep("data.table", localBinsFull, value = TRUE)[1]
+    # localBinsFull <- dts
   } else {
     localBinsFull <- NULL
   }
 
   ## There might be more than one version
-  dts <- grep("data.table", localBinsFull, value = TRUE)[1]
-  localBinsFull <- dts
   Rpath <- Sys.which("Rscript")
   quiet <- !(getOption("Require.verbose") >= 1)
   if (length(localBinsFull) == 1) {
@@ -101,7 +128,7 @@ if (file.exists(pkgVF)) {
   if (is.null(getOption("Require.Home"))) stop("Must define options('Require.Home' = 'pathToRequirePkgSrc')")
   Require:::installRequire(getOption("Require.Home"))
 
-  st <- try(system.time({out <- Require(packageVersionFile = fileNames[["fn0"]][["txt"]])}))
+  out <- Require(packageVersionFile = fileNames[["fn0"]][["txt"]], standAlone = TRUE)
 
   # Test
   there <- data.table::fread(fileNames[["fn0"]][["txt"]])
@@ -115,7 +142,7 @@ if (file.exists(pkgVF)) {
   if (Require:::isWindows())
     anyMissing <- anyMissing[!Package %in% "littler"]
   # here[!there, on = "Package"]
-  if (NROW(anyMissing) != 0) stop("Error 832; please contact developer")
+  if (NROW(anyMissing) != 0)  stop("Error 832; please contact developer")
   testit::assert(NROW(anyMissing) == 0)
 }
 if (!identical(origLibPathsAllTests, .libPaths()))
